
#include <list>

#include "../gsmcomon.h"
#include "../misc/misc.h"

using namespace std;

#ifndef gsmphone_h
#define gsmphone_h

const int ID_None             	 = 1;
const int ID_IncomingFrame       = 2;
const int ID_GetIMEI             = 3;

class GSM_Device;
class GSM_AllProtocols;
class GSM_Protocol_Message;
class GSM_AllPhones;

class GSM_Phone_Info {
public:
        char                    *Protocol;
        char                    *Model;
        char                    *CodeNameModel;
        char                    *DeviceModel;
        char                    *Features;

        GSM_Phone_Info		(char *Mod, char *Code, char *Device, char *Prot, char *Feat);
        ~GSM_Phone_Info		();

	bool 			CompareProtocol(char *Pro);
};

class GSM_Phone
{
        friend class            GSM_AllPhones;
public:
        list <GSM_Phone_Info>   Info;
	char			*ModuleName,*ModulesRequired,*ModulesUsed;

        GSM_Phone		(DebugInfo **Deb, GSM_Device **Dev, GSM_AllProtocols **Prot, GSM_AllPhones **Pho);
        virtual 		~GSM_Phone();

	GSM_Error		Read(int RequestID, void *Struct);
	GSM_Error		Write(unsigned char *buffer, int length, unsigned char type, int time, int request, void *Struct);

        virtual GSM_Error       Dispatch(GSM_Protocol_Message *msg, void *Struct, int RequestID) = 0;

        virtual GSM_Error       Open();
        virtual GSM_Error       Close();
        virtual GSM_Error       GetIMEI(unsigned char *IMEI);
	virtual GSM_Error	GetModel(unsigned char *Model);
	virtual GSM_Error	GetFirmware(unsigned char *Firm);
	virtual GSM_Error	GetCodeNameModel(unsigned char *CodeNameModel);	
protected:
        GSM_Phone               *Next;

        GSM_Device              **Device;
        GSM_AllProtocols        **Protocols;
        GSM_AllPhones           **Phones;
        DebugInfo               **Debug;
	bool			Opened;

	bool			Answer(unsigned char *MsgType, int SubTypeChar, unsigned char SubType, int Request, int RequestID, bool *FrameFound);
};

class GSM_AllPhones
{
	friend class		GSM_Phone;
public:
        GSM_Phone               *Current;

        GSM_AllPhones		(DebugInfo **Deb);
        ~GSM_AllPhones		();

        void                    Add                (GSM_Phone *Phone);
        GSM_Error               RemoveUnused	   ();
        GSM_Error               Switch             (char *Pho, char *Pro);
        GSM_Error               SwitchToDeviceName (char *Pho, char *Pro);
        int                     GetNext            (GSM_Phone **Pho);
private:
        GSM_Phone               *AllPhones;
        DebugInfo               **Debug;

	char			*CFGModel;
	char			*Features;
	char			*Model;
	char			*CodeNameModel;

        unsigned char           ReceivedBuffer[200];
        int                     ReceivedLen, ReceivedPos;

        int                     ReplyNum;
};

#endif
